Package: censusapi
Version: 0.9.0
Title: Retrieve Data from the Census APIs
Authors@R: person("Hannah", "Recht", email = "censusapi.rstats@gmail.com", role = c("aut", "cre"))
Description: A wrapper for the U.S. Census Bureau APIs that returns data frames of 
	Census data and metadata. Available datasets include the 
	Decennial Census, American Community Survey, Small Area Health Insurance Estimates,
	Small Area Income and Poverty Estimates, Population Estimates and Projections, and more.
URL: https://www.hrecht.com/censusapi/,
        https://github.com/hrecht/censusapi
BugReports: https://github.com/hrecht/censusapi/issues
Depends: R (>= 3.5)
License: GPL-3
LazyData: true
RoxygenNote: 7.3.2
Imports: httr, jsonlite, rlang
Suggests: knitr, rmarkdown
Encoding: UTF-8
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2025-03-05 20:42:58 UTC; hrecht
Author: Hannah Recht [aut, cre]
Maintainer: Hannah Recht <censusapi.rstats@gmail.com>
Repository: CRAN
Date/Publication: 2025-03-05 21:10:02 UTC
Built: R 4.3.3; ; 2025-03-05 21:57:30 UTC; unix
