#' List of state fips codes - 50 states plus DC
#'
#' Some small geographies in some Census APIs can only be used under a state hierarchy.
#' This is a list of fips codes that may be looped over to retrieve data for all states.
#' @format A list of fips codes for 50 states and the District of Columbia.
#' @examples
#' fips
"fips"
